@@ -56,7 +56,7 @@ class GaleonHandler(deskbar.interfaces.Module):
 		if is_preferred_browser("galeon"):
 			return True
 		else:
-			print >> sys.stderr, "Galeon is not your preferred browser, not using it."
+			GaleonHandler.INSTRUCTIONS = _("Galeon is not your preferred browser.")
 			return False
 	
 class GaleonBookmarksHandler(GaleonHandler):
