@@ -3,11 +3,17 @@ from gettext import gettext as _
 import gtk
 
 import deskbar.interfaces.Module
+import deskbar.interfaces.Match
 from deskbar.core.DeskbarHistory import DeskbarHistory
 from deskbar.defs import VERSION
 
 HANDLERS = ["HistoryHandler"]
 
+class HistoryMatch(deskbar.interfaces.Match):
+	
+	def __init__(self, name):
+		deskbar.interfaces.Match.__init__(self, name=name, category="history")
+
 class HistoryHandler(deskbar.interfaces.Module):
 	
 	INFOS = {'icon': deskbar.core.Utils.load_icon("stock_redo"),
@@ -35,15 +41,11 @@ class HistoryHandler(deskbar.interfaces.Module):
 		else:
 			inc_priority = lambda x : x + 1
 			
-		for timestamp, text, match in DeskbarHistory.get_instance():
+		for timestamp, text, action in DeskbarHistory.get_instance():
 			if text.startswith(query):
-				match.set_category("history")
+				match = HistoryMatch(action.get_verb() % action.get_escaped_name(query))
+				match.add_action(action)
 				match.set_priority( self.get_priority() + priority )
-				# Spare my life for overriding a private variable, but overriding
-				# the method doesn't work . ie:
-				# match.get_priority = lambda: (self.get_priority(), priority)
-				#match._priority = priority
-				
 				result.append(match)
 				priority = inc_priority (priority)
 		
