@@ -203,10 +203,10 @@ class MozillaSearchHandler(deskbar.interfaces.Module):
 				get_firefox_home_file("searchplugins"),
 				get_firefox_home_file("search"),
 				expanduser("~/.mozilla/searchplugins"),
-				"/usr/lib/firefox/searchplugins",
 				"/usr/local/lib/firefox/searchplugins",
 				"/usr/lib/mozilla-firefox/searchplugins",
-				"/usr/local/lib/mozilla-firefox/searchplugins"]
+				"/usr/local/lib/mozilla-firefox/searchplugins"] + \
+				glob.glob("/usr/lib*/firefox*/searchplugins")
 		else:
 			smart_dirs = [
 				get_mozilla_home_file("search"),
