@@ -19,10 +19,10 @@ class CuemiacHistoryView (gtk.TreeView):
 		title.set_property ("ellipsize", pango.ELLIPSIZE_END)
 		title.set_property ("width-chars", 25) #FIXME: Pick width according to screen size
 		hits = gtk.TreeViewColumn ("Hits")
-		hits.pack_start (icon)
+		hits.pack_start (icon, expand=False)
 		hits.pack_start (title)
-		hits.set_cell_data_func(title, self.__get_match_title_for_cell)			
-		hits.set_cell_data_func(icon, self.__get_match_icon_for_cell)
+		hits.set_cell_data_func(title, self.__get_action_title_for_cell)			
+		hits.set_cell_data_func(icon, self.__get_action_icon_for_cell)
 		self.append_column (hits)
 		
 		self.connect ("row-activated", lambda w,p,c: self.__on_activated())
@@ -31,29 +31,23 @@ class CuemiacHistoryView (gtk.TreeView):
 		self.set_property ("headers-visible", False)
 		self.set_property ("hover-selection", True)
 		
-	def __get_match_icon_for_cell (self, column, cell, model, iter, data=None):
+	def __get_action_icon_for_cell (self, column, cell, model, iter, data=None):
 	
-		timestamp, text, match = model[iter]
-		cell.set_property ("pixbuf", match.get_icon())
-
+		timestamp, text, action = model[iter]
+		if action.get_pixbuf() != None:
+			cell.set_property ("pixbuf", action.get_pixbuf())
 		
-	def __get_match_title_for_cell (self, column, cell, model, iter, data=None):
+	def __get_action_title_for_cell (self, column, cell, model, iter, data=None):
 		
-		timestamp, text, match = model[iter]
-
-		# Pass unescaped query to the matches
-		verbs = {"text" : text}
-		verbs.update(match.get_name(text))
-		# Escape the query now for display
-		verbs["text"] = cgi.escape(verbs["text"])
-		
-		cell.set_property ("markup", match.get_verb () % verbs)
+		timestamp, text, action = model[iter]
+				
+		cell.set_property ("markup", action.get_verb () % action.get_escaped_name(text))
 
 	def __on_activated (self):
 		model, iter = self.get_selection().get_selected()
 		if iter != None:
-			timestamp, text, match = model[iter]
-			self.emit ("match-selected", text, match)
+			timestamp, text, action = model[iter]
+			self.emit ("match-selected", text, action)
 		return False
 
 if gtk.pygtk_version < (2,8,0):	
